@@ -21,17 +21,21 @@ entrez_tool <- function() 'rentrez'
 # This function is used by all the API-querying functions in rentrez to build
 # the appropriate url. Required arguments for each endpoint are handled by
 # specific funcitons. All of these functions can use the id_or_webenv() function
-# (below) to ensure that at least on of these arguments are provided.
+# (below) to ensure that at least on of these arguments are provided and the
+# sleep_time() function to set the approrate time to wait between requests.
 #
+# if debug_mode is set to TRUE the function returns a list with the URL and 
+# arguments that would have been passed to GET or POST (useful for debugging 
+# and used in the test suite).
 
-
-
-
-
-make_entrez_query <- function(util, config, interface=".fcgi?", by_id=FALSE, ...){
+make_entrez_query <- function(util, config, interface=".fcgi?", by_id=FALSE, debug_mode=FALSE, ...){
     uri <- paste0("https://eutils.ncbi.nlm.nih.gov/entrez/eutils/", util, interface)
     args <- list(..., email=entrez_email(), tool=entrez_tool())
-    unsent_flag <- TRUE
+    if(!("api_key" %in% names(args))){ #no api key set, try to use the sytem var
+        if(is_entrez_key_set()){
+            args$api_key <- Sys.getenv('ENTREZ_KEY')
+        }
+    }
     if("id" %in% names(args)){
         if(by_id){
             ids_string <- paste0("id=", args$id, collapse="&")
@@ -40,18 +44,47 @@ make_entrez_query <- function(util, config, interface=".fcgi?", by_id=FALSE, ...
         } else {
             args$id <- paste(args$id, collapse=",")
         }
+    }
+    if(debug_mode){
+        return( list(uri = uri, args=args ) )
+    }
+    
+    #Seemingly, NCBI moved to https but not http v2.0?
+    # (thatnks Jeff Hammerbacher for report/solution)
+    #
+    # if no httr::config was passed we will add one
+    if(is.null(config)){
+         config = httr::config(http_version = 2)
+    # otherwise add http version, checkign we aren't overwriting something
+    # passed in (seems unlikely, but worth checking?)
+    # 
+    } else {
+        if ("http_version" %in% names(config$options)) {
+            warning("Over-writing httr config options for 'http_version', as NCBI servers require v1.1")
+        }
+        config$options$http_version <- 2        
+    }
+
+    
     if(length(args$id) > 200){ 
         response <- httr::POST(uri, body=args, config= config)
-            unsent_flag <- FALSE
-        }
-    }    
-    if (unsent_flag) {
+    } else {
         response <- httr::GET(uri, query=args, config= config) 
     }
     entrez_check(response)
+    Sys.sleep(sleep_time(args))
     httr::content(response, as="text", encoding="UTF-8")
 }
 
+
+#set the sleep time, depending on presence of api_key in the arguments. Used by
+# make_entrez_query
+sleep_time <- function(argument_list){
+    if("api_key" %in% names(argument_list)){
+        return(0.1)
+    }
+    1/3
+}
 ##
 # Check for that we have either the ID or the web-history functions are 
 # specified for those functions that need one.
